import { isString, isNumber } from '@tethys/cdk/is';
import * as i0 from '@angular/core';
import { NgModule } from '@angular/core';

const memoryStorage = new Map();
const cacheHits = new Map();
/**
 * 最大存储的缓存数，超过这个把之前最小命中的缓存去掉
 */
const MAX_CACHE_SIZE = 1000;
class MemoryCache {
    /**
     * set item to local storage
     *
     * @example
     * memoryCache.set('key1', 'key1-value');
     * memoryCache.set('key1', 10);
     * memoryCache.set('key1', { id: 1, name: 'name 1'});
     * memoryCache.set('key1', 'key1-value', false);
     * @param key string
     * @param value string | number | object
     */
    set(key, value) {
        if (!cacheHits.has(key)) {
            cacheHits.set(key, {
                hits: 0
            });
        }
        memoryStorage.set(key, value);
        if (cacheHits.size > MAX_CACHE_SIZE) {
            let minHitKey = '';
            let minHitCount = Number.MAX_SAFE_INTEGER;
            cacheHits.forEach((cacheHit, cacheHitKey) => {
                if (cacheHit.hits < minHitCount) {
                    minHitCount = cacheHit.hits;
                    minHitKey = cacheHitKey;
                }
            });
            if (minHitKey) {
                this.remove(minHitKey);
            }
        }
    }
    /**
     * get item from local storage
     *
     * @example
     * memoryCache.get('key1');
     * memoryCache.get<number>('key1');
     * memoryCache.get<User>('key1');
     * memoryCache.get<User[]>('key1');
     *
     * @param key string
     * @param compress compress data default true
     */
    get(key) {
        let cacheHit = cacheHits.get(key);
        if (!cacheHit) {
            cacheHit = {
                hits: 1
            };
        }
        else {
            cacheHit.hits++;
        }
        cacheHits.set(key, cacheHit);
        return memoryStorage.get(key);
    }
    /**
     * remove key from storage
     * @param key cache key
     */
    remove(key) {
        memoryStorage.delete(key);
        cacheHits.delete(key);
    }
    /**
     * has cache
     * @param key cache key
     */
    has(key) {
        return memoryStorage.has(key);
    }
    /**
     * clear all storage
     */
    clear() {
        memoryStorage.clear();
        cacheHits.clear();
    }
    /**
     * add item to list
     * @example
     * memoryCache.add(`key`, `001`, `value 001`);
     * memoryCache.add(`key`, `001`, { name: 'name 001' });
     * memoryCache.add(`key`, `001`, `value 001`, max);
     */
    listAdd(key, id, value, max = 10) {
        const allItems = this.get(key) || [];
        const currentItem = allItems.find(item => {
            return item.id === id;
        });
        if (!currentItem) {
            allItems.push({
                id: id,
                value: value
            });
            if (allItems.length > max) {
                allItems.splice(0, 1);
            }
        }
        else {
            currentItem.value = value;
        }
        this.set(key, allItems);
    }
    /**
     * get item from list
     * @example
     * memoryCache.get(`key`, `001`);
     */
    listGet(key, id) {
        const allItems = this.get(key) || [];
        const currentItem = allItems.find(item => {
            return item.id === id;
        });
        if (currentItem) {
            return currentItem.value;
        }
        else {
            return null;
        }
    }
    size() {
        return memoryStorage.size;
    }
}
const memoryCache = new MemoryCache();

const NUMBER_PREFIX = '____n____';
const SupportedStorage = window && window.localStorage;
const storageSource = window.localStorage || window.sessionStorage;
const cache = {
    /**
     * set item to local storage
     *
     * @example
     * cache.set('key1', 'key1-value');
     * cache.set('key1', 10);
     * cache.set('key1', { id: 1, name: 'name 1'});
     * cache.set('key1', 'key1-value', false);
     * @param key string
     * @param value string | number | object
     * @param compress compress data default true
     */
    set(key, value) {
        let itemValue = isString(value) ? value : isNumber(value) ? `${NUMBER_PREFIX}${value}` : JSON.stringify(value);
        if (SupportedStorage) {
            storageSource.setItem(key, itemValue);
        }
    },
    /**
     * get item from local storage
     *
     * @example
     * cache.get('key1');
     * cache.get<number>('key1');
     * cache.get<User>('key1');
     * cache.get<User[]>('key1');
     * cache.get('key1', false);
     *
     * @param key string
     */
    get(key) {
        if (SupportedStorage) {
            let value = storageSource.getItem(key);
            if (value) {
                try {
                    const result = JSON.parse(value);
                    return result;
                }
                catch (error) {
                    if (isString(value) && value.includes(NUMBER_PREFIX)) {
                        return parseInt(value.replace(NUMBER_PREFIX, ''), 10);
                    }
                    else {
                        return value;
                    }
                }
            }
            else {
                return undefined;
            }
        }
        else {
            return undefined;
        }
    },
    /**
     * remove key from storage
     * @param key cache key
     */
    remove(key) {
        if (SupportedStorage) {
            storageSource.removeItem(key);
        }
    },
    /**
     * clear all storage
     */
    clear() {
        if (SupportedStorage) {
            storageSource.clear();
        }
    }
};

class TethysCacheModule {
    static { this.ɵfac = i0.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "18.2.0", ngImport: i0, type: TethysCacheModule, deps: [], target: i0.ɵɵFactoryTarget.NgModule }); }
    static { this.ɵmod = i0.ɵɵngDeclareNgModule({ minVersion: "14.0.0", version: "18.2.0", ngImport: i0, type: TethysCacheModule }); }
    static { this.ɵinj = i0.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "18.2.0", ngImport: i0, type: TethysCacheModule }); }
}
i0.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "18.2.0", ngImport: i0, type: TethysCacheModule, decorators: [{
            type: NgModule,
            args: [{}]
        }] });

/**
 * Generated bundle index. Do not edit.
 */

export { TethysCacheModule, cache, memoryCache };
//# sourceMappingURL=cache.mjs.map
